#include "mbdesktop_item.h"
#include "mbdesktop_module.h"

#define WARN(txt) fprintf(stderr, "%s:%i %s %s", __FILE__, __LINE__, __func__, txt )


MBDesktopItem *
mbdesktop_item_new()
{
  MBDesktopItem *ditem;
  ditem = malloc(sizeof(MBDesktopItem));
  memset(ditem, 0, sizeof(MBDesktopItem));
  
  ditem->type = ITEM_TYPE_UNKNOWN;

  return ditem;
}

void
mbdesktop_item_free(MBDesktop     *mb, 
		    MBDesktopItem *item)
{
  /* XXX callback to free item->data */

  if (item->name)      free(item->name);
  if (item->icon_name) free(item->icon_name);
  if (item->icon)      mb_pixbuf_img_free(mb->pixbuf, item->icon);

  /* XXX check for children to free ? */

  free(item);

}

MBDesktopItem*
mbdesktop_item_folder_get_open(MBDesktop     *mb) 
{
  return mb->current_folder_item;
}

MBDesktopItem*
mbdesktop_item_folder_set_view(MBDesktop     *mb,
			       MBDesktopItem *folder,
			       int            view) 
{
  folder->view = view;
  return folder; 		/* XXX ? */
}

int
mbdesktop_item_folder_get_view(MBDesktop     *mb,
			       MBDesktopItem *folder)	
{
  return folder->view;  
}

Bool
mbdesktop_item_folder_has_contents(MBDesktop     *mb, 
				   MBDesktopItem *folder)
{
  if (folder->item_child && folder->item_child->item_next_sibling)
    return True;
  return False;
}

void
mbdesktop_item_folder_contents_free(MBDesktop     *mb, 
				    MBDesktopItem *item)
{
  MBDesktopItem *item_tmp = NULL, *item_cur = NULL;

  if (item->item_child && item->item_child->item_next_sibling)
    {
      if (item == mb->top_head_item )
	{
	  item_cur = item->item_child;
	  mb->top_head_item->item_child = NULL;
	}
      else item_cur = item->item_child->item_next_sibling;

      while (item_cur != NULL)
	{
	  item_tmp = item_cur->item_next_sibling;

	  /* XXX free up any children - check this !! XX */
	  if (item_cur->item_child)
	    mbdesktop_item_folder_contents_free(mb, item_cur->item_child);

	  /* Stop possible segv of focus_item pointing to non-existant */
	  if (mb->kbd_focus_item == item_cur)
	    mb->kbd_focus_item = item->item_child;

	  mbdesktop_item_free(mb, item_cur); 
	  item_cur = item_tmp;
	}
      if ( item != mb->top_head_item)
	item->item_child->item_next_sibling = NULL;
    }
}

MBDesktopItem *
mbdesktop_item_new_with_params (MBDesktop     *mb,
			        char          *name, 
			        char          *icon_name,
			        void          *data,
			        int            type)
{
  MBDesktopItem *ditem;
  ditem = mbdesktop_item_new();

  if (name) ditem->name           = strdup(name);
  if (icon_name)
    {
      ditem->icon_name = strdup(icon_name);
    }

  mbdesktop_item_set_icon_from_theme(mb, ditem);

  if (data) ditem->data           = data;

  /*
  if (exec_str) ditem->exec_str   = strdup(exec_str);
  if (exec_cb) ditem->exec_cb     = exec_cb;
  if (populate_cb) ditem->populate_cb  = populate_cb;
  */

  if (type) ditem->type           = type;

  ditem->view = VIEW_ICONS;

  return ditem;
}

void
mbdesktop_item_set_icon_from_theme (MBDesktop     *mb,
				    MBDesktopItem *item)
{
#ifdef USE_PNG
  char *default_icon_name = "mbnoapp.png";
#else
  char *default_icon_name = "mbnoapp.xpm";
#endif

  char *icon_path = NULL;
  icon_path = mb_dot_desktop_icon_get_full_path (mb->theme_name, 
						 mb->icon_size, 
						 item->icon_name);

  if (icon_path == NULL)
    icon_path = mb_dot_desktop_icon_get_full_path (mb->theme_name, 
						   mb->icon_size, 
						   default_icon_name);

  if (item->icon) mb_pixbuf_img_free(mb->pixbuf, item->icon);

  if ((item->icon = mb_pixbuf_img_new_from_file(mb->pixbuf, icon_path)) 
      == NULL)
    {
      fprintf(stderr, "mbdesktop: could not load %s for %s\n", 
	      icon_path, item->name);
      /* exit(1); */
    }

  if (icon_path) free(icon_path);

  return;
}

void
mbdesktop_items_append (MBDesktop     *mb,
			MBDesktopItem *item_head,
			MBDesktopItem *item )
{
  MBDesktopItem *item_tmp = NULL;

  item_tmp = item_head;

  while ( item_tmp->item_next_sibling != NULL )
    item_tmp = item_tmp->item_next_sibling;

  item_tmp->item_next_sibling = item;
  item->item_prev_sibling = item_tmp;

}

void
mbdesktop_items_append_to_folder (MBDesktop      *mb,
				  MBDesktopItem  *item_folder,
				  MBDesktopItem  *item )
{
  /* -- need to fix this warning --
  if (item_folder->type != ITEM_TYPE_FOLDER && item->type != ITEM_TYPE_ROOT)
    WARN("Passed folder item is not folder\n");
  */

  if (!item_folder->item_child) return;

  item->module_handle = item_folder->module_handle;

  mbdesktop_items_append (mb, item_folder->item_child, item);

}

void
mbdesktop_items_append_to_top_level (MBDesktop      *mb,
				     MBDesktopItem  *item )
{
  MBDesktopItem *top_level = mbdesktop_get_top_level_folder(mb);

  /* item->module_handle = item_folder->module_handle; ???  */ 

  item->item_parent = top_level;

  if (top_level->item_child == NULL)
    top_level->item_child = item;
  else
    mbdesktop_items_append (mb, top_level->item_child, item); 
}

void
mbdesktop_items_prepend (MBDesktop      *mb,
			 MBDesktopItem **item_head,
			 MBDesktopItem  *item )
{
  MBDesktopItem *item_tmp = NULL;

  /*
  if ((*item_head)->type == ITEM_TYPE_PREVIOUS)
    item_tmp = (*item_head)->item_next_sibling;
  else
  */

  item_tmp = *item_head;
  item->item_next_sibling = item_tmp;
  item_tmp->item_prev_sibling = item;

  *item_head = item;


}

MBDesktopItem *
mbdesktop_item_get_next_sibling(MBDesktopItem *item)
{
  return item->item_next_sibling;
}

MBDesktopItem *
mbdesktop_item_get_prev_sibling(MBDesktopItem *item)
{
  return item->item_prev_sibling;
}

MBDesktopItem *
mbdesktop_item_get_parent(MBDesktopItem *item)
{
  MBDesktopItem *result = mbdesktop_item_get_first_sibling(item);

  if (result && result->item_parent)
    return result->item_parent;

  return NULL;
}

MBDesktopItem *
mbdesktop_item_get_child(MBDesktopItem *item)
{
  /* XXX Need to fix warnings
  if (item->type != ITEM_TYPE_FOLDER && item->type != ITEM_TYPE_ROOT)
    WARN("Passed folder item is not folder\n");
  */

  return item->item_child;
}


MBDesktopItem *
mbdesktop_item_get_first_sibling(MBDesktopItem *item)
{
  while (item->item_prev_sibling != NULL )
    item = item->item_prev_sibling;
  return item;
}

void
mbdesktop_item_set_icon_data (MBDesktop     *mb, 
			      MBDesktopItem *item, 
			      MBPixbufImage *img)
{
  if (item->icon) mb_pixbuf_img_free(mb->pixbuf, item->icon);
  item->icon = mb_pixbuf_img_clone(mb->pixbuf, img);
}

void
mbdesktop_item_set_name (MBDesktop     *mb, 
			 MBDesktopItem *item, 
			 char          *name)
{
  if (item->name) free(item->name);
  item->name = strdup(name);
}

char *
mbdesktop_item_get_name (MBDesktop     *mb, 
			 MBDesktopItem *item)
{
  return item->name;
}

void
mbdesktop_item_set_user_data (MBDesktop     *mb, 
			      MBDesktopItem *item, 
			      void          *data)
{
  if (item->data) free(item->data); /* XXX callback ? */
  item->data = data;
}

void *
mbdesktop_item_get_user_data (MBDesktop     *mb, 
			      MBDesktopItem *item)
{
  return item->data;
}


void
mbdesktop_item_set_image (MBDesktop     *mb, 
			  MBDesktopItem *item, 
			  char          *full_img_path)
{
  if (item->icon) mb_pixbuf_img_free(mb->pixbuf, item->icon);
  item->icon = mb_pixbuf_img_new_from_file(mb->pixbuf, full_img_path);
}

void
mbdesktop_item_set_activate_callback (MBDesktop     *mb, 
				      MBDesktopItem *item, 
				      MBDesktopCB    activate_cb)
{
  item->activate_cb = activate_cb;
}

void
mbdesktop_item_set_type (MBDesktop     *mb, 
			 MBDesktopItem *item,
			 int            type)
{
  item->subtype = type;
}

int
mbdesktop_item_get_type (MBDesktop     *mb, 
			 MBDesktopItem *item)
{
  return item->subtype;
}


/* Should these live here ?? */

void
mbdesktop_item_folder_activate_cb(void *data1, void *data2)
{
  MBDesktop *mb = (MBDesktop *)data1; 
  MBDesktopItem *item = (MBDesktopItem *)data2; 

  mb->scroll_offset_item = mb->kbd_focus_item 
    = mb->current_head_item = item->item_child;

  mb->current_folder_item = item; 

  /* XXX set current folder item, for module call backs */

  /*
  if (item->module_handle)
    {
      if (item->module_handle->folder_open)
	item->module_handle->folder_open(mb, item);
    }
  */

  mbdesktop_view_paint(mb, False);
}

void
mbdesktop_item_folder_prev_activate_cb(void *data1, void *data2)
{
  MBDesktop *mb = (MBDesktop *)data1; 
  MBDesktopItem *item = (MBDesktopItem *)data2; 

  mb->scroll_offset_item = mb->kbd_focus_item 
    = mb->current_head_item = item->item_child;

  mb->kbd_focus_item = item->item_parent;
  mb->scroll_offset_item  = mb->current_head_item 
    = mbdesktop_item_get_first_sibling(item->item_parent);

  mb->current_folder_item = item->item_parent->item_parent; 

  /* XXX not needed anymore
  if (item->item_parent->module_handle)
    {
      if (item->item_parent->module_handle->folder_open)
	item->item_parent->module_handle->folder_close(mb, item->item_parent);
    }
  */

  mbdesktop_view_paint(mb, False);
}

MBDesktopItem *
mbdesktop_item_get_from_coords(MBDesktop *mb, int x, int y)
{
  MBDesktopItem *item;
  for(item = mb->scroll_offset_item; 
      (item != NULL && item != mb->last_visible_item); 
      item = item->item_next_sibling)
    {
      if (x > item->x && x < (item->x + item->width)
	  && y > item->y && y < (item->y + item->height))
	{
	  return item;
	} 
    }
  return NULL;
}
