#include "mbdesktop_view.h"
#ifdef PANGO
#include <pango/pango.h>
#include <pango/pangoxft.h>
#endif

static void
mbdesktop_view_paint_items(MBDesktop *mb, MBPixbufImage *img_dest);


void
mbdesktop_view_init_bg(MBDesktop *mb)
{
  MBPixbufImage *img_tmp;
  int dw, dh, dx, dy, r, g, b;
  char *p;

  if (mb->bg_img != NULL)
    mb_pixbuf_img_free(mb->pixbuf, mb->bg_img);

  switch (mb->bg->type)
    {
    case BG_SOLID:
      mb->bg_img = mb_pixbuf_img_rgba_new(mb->pixbuf, mb->desktop_width, 
					 mb->desktop_height);
      mb_pixbuf_img_fill(mb->pixbuf, mb->bg_img, 
			 mb->bg->data.cols[0], 
			 mb->bg->data.cols[1],
			 mb->bg->data.cols[2], 
			 0);


      mb->use_text_outline = False;

      if (!mb->user_overide_font_col)
	{
	  if ((((mb->bg->data.cols[0] * 54) + (mb->bg->data.cols[1] * 183) + (mb->bg->data.cols[2] * 19)) / 256) > 127 )
	    {
	      mbdesktop_set_font_color(mb, "black");
	    } else {
	      mbdesktop_set_font_color(mb, "white");
	    }
	}

      break;
    case BG_TILED_PXM:
      mb->bg_img = mb_pixbuf_img_rgb_new(mb->pixbuf, mb->desktop_width, 
					 mb->desktop_height);

      if ((img_tmp = mb_pixbuf_img_new_from_file(mb->pixbuf, 
						 mb->bg->data.filename)) 
	  == NULL)
	{
	  fprintf(stderr,"Failed to load background : %s", 
		  mb->bg->data.filename);
	  mbdesktop_bg_parse_spec(mb, "col-solid:red");
	  mbdesktop_view_init_bg(mb);
	  return;
	}

      for (dy=0; dy < mb->desktop_height; dy += img_tmp->height)
	for (dx=0; dx < mb->desktop_width; dx += img_tmp->width)
	  {
	    if ( (dx + img_tmp->width) > mb->desktop_width )
	      dw = img_tmp->width - ((dx + img_tmp->width)-mb->desktop_width);
	    else
	      dw = img_tmp->width;

	    if ( (dy + img_tmp->height) > mb->desktop_height )
	      dh = img_tmp->height-((dy + img_tmp->height)-mb->desktop_height);
	    else
	      dh = img_tmp->height;
	    mb_pixbuf_img_copy(mb->pixbuf, mb->bg_img, img_tmp,
			       0, 0, dw, dh, dx, dy);
	  }
      mb_pixbuf_img_free(mb->pixbuf, img_tmp);

      if (!mb->user_overide_font_outline) mb->use_text_outline = True;
      if (!mb->user_overide_font_col) mbdesktop_set_font_color(mb, "white");

      break;
    case BG_STRETCHED_PXM:
      if ((img_tmp = mb_pixbuf_img_new_from_file(mb->pixbuf, 
						 mb->bg->data.filename)) 
	  == NULL)
	{
	  fprintf(stderr,"Failed to load background : %s", mb->bg->data.filename);
	  mbdesktop_bg_parse_spec(mb, "col-solid:red");
	  mbdesktop_view_init_bg(mb);
	  return;
	}
      mb->bg_img = mb_pixbuf_img_scale(mb->pixbuf, img_tmp, mb->desktop_width, 
				       mb->desktop_height);
      mb_pixbuf_img_free(mb->pixbuf, img_tmp);

      if (!mb->user_overide_font_outline) mb->use_text_outline = True;
      if (!mb->user_overide_font_col) mbdesktop_set_font_color(mb, "white");

      break;
    case BG_CENTERED_PXM:
      if ((img_tmp = mb_pixbuf_img_new_from_file(mb->pixbuf, 
						 mb->bg->data.filename)) 
	  == NULL)
	{
	  fprintf(stderr,"Failed to load background : %s", mb->bg->data.filename);
	  mbdesktop_bg_parse_spec(mb, "col-solid:red");
	  mbdesktop_view_init_bg(mb);
	  return;
	}
      dx = (mb->desktop_width - img_tmp->width) / 2;
      if (dx < 0) dx = 0;
      dy = (mb->desktop_height - img_tmp->height) / 2;
      if (dy < 0) dy = 0;
      mb->bg_img = mb_pixbuf_img_rgb_new(mb->pixbuf, mb->desktop_width, 
				     mb->desktop_height);
      mb_pixbuf_img_copy(mb->pixbuf, mb->bg_img, img_tmp,
			 0, 0,
			 (img_tmp->width > mb->desktop_width) ?
			 mb->desktop_width : img_tmp->width ,
			 (img_tmp->height > mb->desktop_height) ?
			 mb->desktop_height : img_tmp->height ,
			 dx, dy);
      mb_pixbuf_img_free(mb->pixbuf, img_tmp);

      if (!mb->user_overide_font_outline) mb->use_text_outline = True;
      if (!mb->user_overide_font_col) mbdesktop_set_font_color(mb, "white");

      break;
    case BG_GRADIENT_HORIZ:
      mb->bg_img = mb_pixbuf_img_rgb_new(mb->pixbuf, mb->desktop_width, 
					 mb->desktop_height);
      p = mb->bg_img->rgba;

      dw = mb->desktop_width;
      dh = mb->desktop_height;
      
      for(dx=0; dx<dw; dx++)
	{
	  r = mb->bg->data.gcols[0] + (( dx * (mb->bg->data.gcols[1] - mb->bg->data.gcols[0])) / dw); 
	  g = mb->bg->data.gcols[2] + (( dx * (mb->bg->data.gcols[3] - mb->bg->data.gcols[2])) / dw); 
	  b = mb->bg->data.gcols[4] + (( dx * (mb->bg->data.gcols[5] - mb->bg->data.gcols[4])) / dw); 

	  for(dy=0; dy<dh; dy++)
	    {
	      *p++ = r;
	      *p++ = g;
	      *p++ = b;
	    }
	}

      if (!mb->user_overide_font_outline) mb->use_text_outline = True;
      if (!mb->user_overide_font_col) mbdesktop_set_font_color(mb, "white");

      break;
    case BG_GRADIENT_VERT:
      mb->bg_img = mb_pixbuf_img_rgb_new(mb->pixbuf, mb->desktop_width, 
					 mb->desktop_height);
      p = mb->bg_img->rgba;

      dw = mb->desktop_width;
      dh = mb->desktop_height;
      
      for(dy=0; dy<dh; dy++)
	{
	  r = mb->bg->data.gcols[0] + (( dy * (mb->bg->data.gcols[1] - mb->bg->data.gcols[0])) / dh); 
	  g = mb->bg->data.gcols[2] + (( dy * (mb->bg->data.gcols[3] - mb->bg->data.gcols[2])) / dh); 
	  b = mb->bg->data.gcols[4] + (( dy * (mb->bg->data.gcols[5] - mb->bg->data.gcols[4])) / dh); 

	  for(dx=0; dx<dw; dx++)
	    {
	      *p++ = r;
	      *p++ = g;
	      *p++ = b;
	    }
	}

      if (!mb->user_overide_font_outline) mb->use_text_outline = True;
      if (!mb->user_overide_font_col) mbdesktop_set_font_color(mb, "white");

      break;
    }

  /* Now set the background */
  mbdesktop_view_set_root_pixmap(mb, mb->bg_img);

  /* and figure out what font is best */

}

void
mbdesktop_view_set_root_pixmap(MBDesktop *mb, MBPixbufImage *img)
{
  Atom atom_root_pixmap_id = XInternAtom(mb->dpy, "_XROOTPMAP_ID", False); 
  
  if (mb->root_pxm != None) XFreePixmap(mb->dpy, mb->root_pxm);
  
  mb->root_pxm = XCreatePixmap(mb->dpy, mb->root, 
			       img->width, img->height,
			       mb->pixbuf->depth ); 

  mb_pixbuf_img_render_to_drawable(mb->pixbuf, img, (Drawable)mb->root_pxm,
				   0, 0);

  XChangeProperty(mb->dpy, mb->root, atom_root_pixmap_id, XA_PIXMAP, 
		  32, PropModeReplace, (unsigned char *) &mb->root_pxm, 1);
}


void  /* called only when desktop/workarea changes size or bgimage changes */
mbdesktop_view_configure(MBDesktop *mb)
{
  /* Assume mb->desktop_width, etc is up to date */

  int x,y,w,h;

  /* This is a bit crap - probably already called */
  if (mbdesktop_get_workarea(mb, &x, &y, &w, &h))
    {
      mb->workarea_x = x; 
      mb->workarea_y = y ;
      mb->workarea_width = w;
      mb->workarea_height = h;
    }

  mbdesktop_view_init_bg(mb); 	   /* reset the background */
  mbdesktop_view_paint(mb, False); /* repaint */

}


void  /* TODO: implement multiple views */
mbdesktop_set_view(MBDesktop *mb, int view)
{
  ;
}

void 
mbdesktop_view_paint_item_text (MBDesktop     *mb, 
				MBDesktopItem *item,
#ifdef USE_XFT
				XftDraw        *dest,
#else
				Drawable       dest,
#endif
				int            x,
				int            y,
				int            align)
{
  int align_offset   = 0;
  int name_txt_width = 0;
  int avail_width    = item->width; 
  char *name         = strdup(item->name);

  /* XXX layout code Works nice, but needs a CLEAN UP XXX */

#ifdef USE_XFT        
  XGlyphInfo extents;
  XftTextExtentsUtf8(mb->dpy, mb->font, (unsigned char *)name,
		     strlen(name), &extents);

  name_txt_width = extents.width;
#else
  XSetFont(mb->dpy, mb->gc, mb->font->fid);
  name_txt_width = XTextWidth(mb->font, name, strlen(name));
#endif

  if (mbdesktop_current_folder_view(mb) == VIEW_LIST)
    avail_width -= mb->icon_size;

  if (name_txt_width > avail_width) /* Text is too wide */
    {
      char *p = name;
      char *q = p;
      char *backtrack = NULL;
      int   v_offset  = 0;
      int   cur_width = 0;
      int   i = 0; 		/* Current line, max 2 lines */


      while (*p != '\0' && i < 2)
	{                       /* Parse text till we hit a space */
	  if (isspace(*p) || *(p+1) == '\0')
	    {
	      Bool is_end = False;

	      if (*(p+1) == '\0') /* Are we at the end of the text ? */
		{
		  is_end = True;
		}
	      else *p = '\0';
#ifdef USE_XFT
	      XftTextExtentsUtf8(mb->dpy, mb->font, (unsigned char *)q,
				 strlen(q), &extents);
	      
	      cur_width += extents.width;
	      
	      if (cur_width > avail_width || is_end)
		{
		  if (backtrack != NULL)
		    {
		      *backtrack = '\0';
		      p = backtrack + 1;
		      XftTextExtentsUtf8(mb->dpy, mb->font, (unsigned char *)q,
					 strlen(q), &extents);
		      cur_width = 0;
		    }
		  else if ( cur_width > avail_width )
		    {
		      /* We cant backtrack to previous word, so we just clip
			 the text.
		      */
		      int num_chars = strlen(q);

		      XftTextExtentsUtf8(mb->dpy, mb->font, (unsigned char *)q,
					 num_chars, &extents);
		      
		      while (extents.width > avail_width)
			{
			  num_chars--;
			  XftTextExtentsUtf8(mb->dpy, mb->font, 
					     (unsigned char *)q,
					     num_chars, &extents);
			}
		      if (num_chars > 2)
			{
			  *(q + num_chars - 2) = '.';
			  *(q + num_chars - 1) = '.';
			}

		      *(q + num_chars)     = '\0';
		    }
		}
	      else
		{
		  /* Can render anything yet. Just store the backtrack 
		     position and carry on
		  */

		  *p = ' ';
		  backtrack = p;
		  p++;
		  continue;
		}
	      
	      /* Now do the actual rendering */
	      
	      if (align == ALIGN_CENTER) 	/* Center */
		align_offset = (( avail_width - extents.width)/2);
	      else
		align_offset = 0;
	      
	      if (mb->use_text_outline)
		XftDrawStringUtf8(dest, 
				  &mb->xftbgcol, 
				  mb->font,
				  x + align_offset + 1, 
				  y + v_offset + 1 + mb->font->ascent,
				  (unsigned char *)q, 
				  strlen(q)
				  );
	      
	      XftDrawStringUtf8(dest, 
				&mb->xftcol, 
				mb->font,
				x + align_offset, 
				y + v_offset + mb->font->ascent,
				(unsigned char *)q, 
				strlen(q)
				);
	      v_offset += mb->font->ascent + mb->font->descent;
#else
	      name_txt_width = XTextWidth(mb->font, q, strlen(q));
	      XDrawString(mb->dpy, dest, mb->gc,
			  x + align_offset, 
			  y + v_offset + mb->font->ascent,
			  (unsigned char *)q, 
			  strlen(q)
			  );
	      v_offset += mb->font->ascent + mb->font->descent;
#endif
	      q= p;
	      backtrack = NULL;
	      //p = q;
	      i++;
	    }
	  p++;
	}
    }
  else 
    {
#ifdef USE_XFT

      if (align == ALIGN_CENTER) 	/* Center */
	align_offset = (( avail_width - extents.width)/2);
      else
	align_offset = 0;

      if (mb->use_text_outline)
	XftDrawStringUtf8(dest, 
			  &mb->xftbgcol, 
			  mb->font,
			  x + align_offset + 1, 
			  y + mb->font->ascent + 1,
			  (unsigned char *)name, 
			  strlen(name)
			  );
      
      XftDrawStringUtf8(dest, 
			&mb->xftcol, 
			mb->font,
			x + align_offset, 
			y + mb->font->ascent,
			(unsigned char *)name, 
			strlen(name)
			);
#else
      XDrawString(mb->dpy, dest /*mb->backing_cache*/, mb->gc,
		  x + align_offset, 
		  y + mb->font->ascent,
		  (unsigned char *)name, strlen(name));
      
#endif
    }

  free(name);
}


void 
mbdesktop_view_paint(MBDesktop *mb, Bool use_cache)
{
  MBPixbufImage *img_dest;
  char *folder_title = "Home";

  if (use_cache && mb->backing_cache != None)
    {
      if (mb->had_kbd_input && mb->kbd_focus_item)
	{
	  if (mb->have_focus)
	    mbdesktop_view_item_highlight (mb, mb->kbd_focus_item, 
					   HIGHLIGHT_OUTLINE); 
	}
      return;
    }

  if (mb->backing_cache != None)
    XFreePixmap(mb->dpy, mb->backing_cache);

  mb->backing_cache = XCreatePixmap(mb->dpy, mb->root, 
				    mb->desktop_width, mb->desktop_height,
				    mb->pixbuf->depth ); 

#ifdef USE_XFT  
  if (mb->xftdraw != NULL)
    XftDrawDestroy (mb->xftdraw);

  mb->xftdraw = XftDrawCreate(mb->dpy, (Drawable) mb->backing_cache, 
			      mb->pixbuf->vis,
			      mb->pixbuf->root_cmap
			      );
#endif

  img_dest = mb_pixbuf_img_clone(mb->pixbuf, mb->bg_img);

  if ( mbdesktop_current_folder_view (mb) == VIEW_ICONS )
    mbdesktop_view_paint_items(mb, img_dest);
  else
    mbdesktop_view_paint_list(mb, img_dest);

  if (mb->current_head_item->item_parent)
    folder_title = mb->current_head_item->item_parent->name;
  else 
    folder_title = mb->top_head_item->name;

  XSetLineAttributes(mb->dpy, mb->gc, 1, LineSolid, CapRound, JoinRound);

#ifdef USE_XFT  

  if (mb->use_text_outline)
    XftDrawStringUtf8(mb->xftdraw, 
		      &mb->xftbgcol, 
		      mb->titlefont,
		      mb->workarea_x + ((48-32)/2) + 1,
		      mb->workarea_y + mb->titlefont->ascent + 1,
		      (unsigned char *)folder_title,
		      strlen(folder_title)
		      );

  XftDrawStringUtf8(mb->xftdraw, 
		    &mb->xftcol, 
		    mb->titlefont,
		    mb->workarea_x + ((48-32)/2),
 		    mb->workarea_y + mb->titlefont->ascent,
		    (unsigned char *)folder_title,
		    strlen(folder_title)
		    );

  XDrawLine(mb->dpy, mb->backing_cache, mb->gc, 
	    mb->workarea_x + ((48-32)/2),
	    mb->workarea_y + mb->titlefont->ascent 
	    + mb->titlefont->descent + 2,
	    mb->workarea_x + mb->workarea_width - ((48-32)/2),
	    mb->workarea_y + mb->titlefont->ascent 
	    + mb->titlefont->descent + 2);

#else

   XSetFont(mb->dpy, mb->gc, mb->titlefont->fid);
   XDrawString(mb->dpy, mb->backing_cache, mb->gc,
	       mb->workarea_x + ((48-32)/2),
	       mb->workarea_y + mb->titlefont->ascent,
	       (unsigned char *)folder_title,
	       strlen(folder_title));

   XDrawLine(mb->dpy, mb->backing_cache, mb->gc, 
	     mb->workarea_x + ((48-32)/2),
	     mb->workarea_y + mb->titlefont->ascent 
	     + mb->titlefont->descent + 2,
	     mb->workarea_x + mb->workarea_width - ((48-32)/2),
	     mb->workarea_y + mb->titlefont->ascent 
	     + mb->titlefont->descent + 2);

#endif


  XSetLineAttributes(mb->dpy, mb->gc, 1, LineOnOffDash, CapRound, JoinRound);

  XSetWindowBackgroundPixmap(mb->dpy, mb->win_top_level, mb->backing_cache);
  XClearWindow(mb->dpy, mb->win_top_level);

  if (mb->kbd_focus_item && mb->had_kbd_input)
    {
      if (mb->have_focus)
	mbdesktop_view_item_highlight (mb, mb->kbd_focus_item, 
				       HIGHLIGHT_OUTLINE); 
    }

  if (img_dest)
    mb_pixbuf_img_free(mb->pixbuf, img_dest);  
}

void
mbdesktop_view_paint_list(MBDesktop *mb, MBPixbufImage *dest_img)
{
  /* 
     XXX TODO
       - paint correct icon size

  */

  MBDesktopItem *item;
  MBPixbufImage *icon_img_small;
  int cur_y, cur_x, limit_y;

  if (mb->scroll_offset_item  == mb->current_head_item)
    mb->scroll_active = False;
  else
    mb->scroll_active = True;


  cur_x = mb->workarea_x + ((48-32)/2);
  cur_y = mb->workarea_y + mb->title_offset;
  limit_y = mb->workarea_y + mb->workarea_height;

  for(item = mb->scroll_offset_item; 
      item != NULL; 
      item = item->item_next_sibling)
    {
      if (item->icon)
	{
	  if ( (cur_y + mb->icon_size ) > limit_y) /* Off display ? */
	    {
	      mb->scroll_active = True;
	      break;
	    }

	  icon_img_small = mb_pixbuf_img_scale(mb->pixbuf, 
					       item->icon,
					       mb->icon_size, mb->icon_size);

	  mb_pixbuf_img_composite(mb->pixbuf, dest_img, 
				  icon_img_small, 
				  cur_x, 
				  cur_y );

	  mb_pixbuf_img_free(mb->pixbuf, icon_img_small);
	  item->x = cur_x;
	  item->y = cur_y;
	  item->width  = mb->workarea_width - ((48-32));
	  item->height = mb->icon_size;

	  cur_y += mb->icon_size;

	}
    }

  if (mb->scroll_active)
    {
      mb_pixbuf_img_composite(mb->pixbuf, dest_img, mb->img_scroll_up,
			      mb->workarea_x + mb->workarea_width-24, 
			      mb->workarea_y + 2);
    
      mb_pixbuf_img_composite(mb->pixbuf, dest_img,
			      mb->img_scroll_down, 
			      mb->workarea_x + mb->workarea_width-40, 
			      mb->workarea_y + 2);

    }


  mb_pixbuf_img_render_to_drawable(mb->pixbuf, dest_img, 
				   (Drawable)mb->backing_cache, 
				   0, 0);

  mb->last_visible_item = item;


  for(item = mb->scroll_offset_item; 
      item != NULL; 
      item = item->item_next_sibling)
    {
      int foffsety = 0;

      foffsety = (mb->icon_size -(mb->font->ascent+mb->font->descent))/2;
      if (foffsety < 0) foffsety = 0;

      if (item == mb->last_visible_item)
	break;

      mbdesktop_view_paint_item_text(mb, item, 
#ifdef USE_XFT
				     mb->xftdraw,
#else
				     mb->backing_cache,
#endif				     
				     item->x + mb->icon_size, 
				     item->y + foffsety,
				     ALIGN_LEFT); 

    }

}


static void
mbdesktop_view_paint_items(MBDesktop *mb, MBPixbufImage *img_dest)
{
  MBDesktopItem *item;
  MBPixbufImage *icon_img_small;

  int cur_x = 0, cur_y = 0, limit_x, limit_y;
  int item_horiz_border = (mb->item_width-(mb->icon_size))/2;

  if (mb->scroll_offset_item  == mb->current_head_item)
    mb->scroll_active = False;
  else
    mb->scroll_active = True;

  cur_x = mb->workarea_x;
  cur_y = mb->workarea_y + mb->title_offset;

  limit_x = mb->workarea_x + mb->workarea_width;
  limit_y = mb->workarea_y + mb->workarea_height + mb->title_offset;

  for(item = mb->scroll_offset_item; 
      item != NULL; 
      item = item->item_next_sibling)
    {
      if (item->type == ITEM_TYPE_MODULE_WINDOW)
	{
	  /* 
	     May need to 'newline' thigs here.

	  */

	    item->x      = mb->workarea_x;
	    item->y      = cur_y;
	    item->width  = mb->workarea_width;
	    /* item->height stays the same
	    
	    mbdesktop_xembed_configure (mb, item);

	    */

	    cur_x = mb->workarea_x;
	    cur_y += mb->item_height;


	    /*
	     Its a window. 
	       - Move it ?
	       - Is it offscreen ?
	       - resize its width if changed ?
	       - send it an expose ?
	       - cur_y + window height. 
	    */
	}
      else
	{
	  if ((cur_x + mb->item_width) > limit_x) /* 'newline' */
	    {
	      cur_x = mb->workarea_x;
	      cur_y += mb->item_height;

	      if ( (cur_y+mb->item_height) > limit_y) /* Off display ? */
		{
		  /* 
		     set a flag to turn on scrolling
		  */
		  mb->scroll_active = True;
		  break;
		}
	    }

	  item->x      = cur_x;
	  item->y      = cur_y;
	  item->width  = mb->item_width;
	  item->height = mb->item_height;
	  
	  if (item->icon)
	    {
	      if (mb_pixbuf_img_get_width(item->icon) != mb->icon_size
		  || mb_pixbuf_img_get_height(item->icon) != mb->icon_size)
		{
		  icon_img_small = mb_pixbuf_img_scale(mb->pixbuf, item->icon,
						       mb->icon_size, 
						       mb->icon_size);
		  
		  mb_pixbuf_img_composite(mb->pixbuf, img_dest, 
					  icon_img_small, 
					  item->x + item_horiz_border, 
					  item->y );
		  
		  mb_pixbuf_img_free(mb->pixbuf, icon_img_small);
		  
		} else mb_pixbuf_img_composite(mb->pixbuf, img_dest, 
					       item->icon, 
					       item->x + item_horiz_border, 
					   item->y );
	    }
	  
	  cur_x += mb->item_width;
	}
    }

  if (mb->scroll_active)
    {
      mb_pixbuf_img_composite(mb->pixbuf, img_dest, mb->img_scroll_up,
			      mb->workarea_x + mb->workarea_width-24, 
			      mb->workarea_y + 2);
    
      mb_pixbuf_img_composite(mb->pixbuf, img_dest, mb->img_scroll_down, 
			      mb->workarea_x + mb->workarea_width-40, 
			      mb->workarea_y + 2);

    }


  mb_pixbuf_img_render_to_drawable(mb->pixbuf, img_dest, 
				   (Drawable)mb->backing_cache, 
				   0, 0);

  mb->last_visible_item = item;

  /* 2nd pass render text */

  for(item = mb->scroll_offset_item; 
      (item != NULL && item != mb->last_visible_item); 
      item = item->item_next_sibling)
    {
      mbdesktop_view_paint_item_text(mb, item, 
#ifdef USE_XFT
				     mb->xftdraw,
#else
				     mb->backing_cache,
#endif				     
				     item->x, item->y + mb->icon_size,
				     ALIGN_CENTER); 
    }
}

void
mbdesktop_view_item_highlight (MBDesktop     *mb, 
			       MBDesktopItem *item,
			       int            highlight_style)
{
  MBPixbufImage *img_cache = NULL;
  Pixmap pxm;

#ifdef USE_XFT  
  XftDraw *xftdraw;
#endif

  int align = ALIGN_LEFT;
  int text_y_offset = 0;
  int cur_view = 0;
  int x = 0, y = 0, w = 0, h = 0, xx = 0, yy = 0;

  cur_view = mbdesktop_current_folder_view(mb);

  switch (cur_view)
    {
    case VIEW_ICONS:
      x = item->x;
      y = item->y      + mb->icon_size;
      w = item->width;
      h = item->height - mb->icon_size;
      align = ALIGN_CENTER;

      break;
    case VIEW_LIST:
      x = item->x + mb->icon_size;
      y = item->y;
      w = item->width - mb->icon_size;
      h = item->height;

      text_y_offset = (mb->icon_size - (mb->font->ascent+mb->font->descent))/2;
      if (text_y_offset < 0) text_y_offset = 0;

      align = ALIGN_LEFT;
      break;
    }
  
  switch (highlight_style)
    {
    case HIGHLIGHT_OUTLINE:
    case HIGHLIGHT_FILL:

      /* XXX was draw 
      XFillRectangle(mb->dpy, mb->win_top_level, mb->invert_gc, x, y, w, h);
      */
      
      img_cache = mb_pixbuf_img_rgba_new(mb->pixbuf, w, h);
      
      mb_pixbuf_img_copy (mb->pixbuf, img_cache, mb->bg_img,
			  x, y, w, h, 0, 0);

      for ( xx=2; xx < (w - 2); xx++)
	{
	  mb_pixbuf_img_plot_pixel(mb->pixbuf, img_cache, xx, 0,
				   0x80, 0x80, 0x80);
	  mb_pixbuf_img_plot_pixel(mb->pixbuf, img_cache, xx, h-2,
				   0x80, 0x80, 0x80);
	}
      
      for ( xx=1; xx < (w - 1); xx++)
	{
	  mb_pixbuf_img_plot_pixel(mb->pixbuf, img_cache, xx, 1,
				   0x80, 0x80, 0x80);
	  mb_pixbuf_img_plot_pixel(mb->pixbuf, img_cache, xx, h-3,
				   0x80, 0x80, 0x80);
	}
      
      for ( xx=0; xx < w; xx++)
	for ( yy=2; yy < h-3; yy++)
	  mb_pixbuf_img_plot_pixel(mb->pixbuf, img_cache, xx, yy,
				   0x80, 0x80, 0x80);
      
      pxm = XCreatePixmap(mb->dpy, mb->root, w, h, mb->pixbuf->depth ); 

#ifdef USE_XFT  
      xftdraw = XftDrawCreate(mb->dpy, (Drawable)pxm, mb->pixbuf->vis,
			      mb->pixbuf->root_cmap);
#endif

      mb_pixbuf_img_render_to_drawable(mb->pixbuf, img_cache, 
				       (Drawable)pxm, 0, 0);
  
      mbdesktop_view_paint_item_text(mb, item,
#ifdef USE_XFT  
				     xftdraw,
#else
				     pxm,
#endif
				     0, text_y_offset,
				     align); 
      
      XCopyArea(mb->dpy, pxm, mb->win_top_level, mb->gc, 0, 0, w, h, x, y);
  
      mb_pixbuf_img_free(mb->pixbuf, img_cache);
#ifdef USE_XFT
      XftDrawDestroy (xftdraw);
#endif
      XFreePixmap(mb->dpy, pxm);

      break;

    case HIGHLIGHT_OUTLINE_CLEAR:
    case HIGHLIGHT_FILL_CLEAR:
      XClearWindow(mb->dpy, mb->win_top_level);


      // XClearWindow(mb->dpy, mb->win_top_level);

      /* XFillRectangle(mb->dpy, mb->win_top_level, mb->invert_gc, x, y, w, h);*/
      
      break;
    }

  XFlush(mb->dpy);
}

